﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace IK
{
    class ARCChain : Chain
    {
        public ARCChain(int count)
            : base(count)
        { }


        public override void Move(Target target, int max_iterations, double tolerance)
        {
            bool reachable = false;
            bool reached = false;
            int iterations = 0;
            double effort = 0;
            double distance = 0;
            double netEffort = 0;


            reachable = (links.Sum(f => f.Length)) + tolerance >= Length();


            double n = links.Length;
            double a = links[0].Length;
            double d = target.Point.Length();
            double semi = a / (2.0 * Math.Sin(Math.PI / (2.0 * n)));


            // Use the newton raphson method to converge to a solution

            double theta = (2.0 * Math.PI) / n; // initial guess

            for (int i = 0; i < max_iterations; i++) {
                double f = d * Math.Sin(theta / 2.0) - a * Math.Sin(theta * n / 2.0);
                double ff = d / 2.0 * Math.Cos(theta / 2.0) - a * n / 2.0 * Math.Cos(theta * n / 2.0);

                theta -= f / ff;

                // Update sample
                iterations++;

                // early terminiation ???

                // Update sample
                reached = Math.Abs(a * Math.Sin(theta * n / 2.0) / Math.Sin(theta / 2.0) - d) <= tolerance;
                if (reached) break;
            }
            //theta = -theta; // reverse the arc direction


            // Update the links
            links[0].Angle = 2.0 * (Math.PI - theta) + target.Point.Angle();
            for (int i = 1; i < links.Length; i++) {
                links[i].Angle = theta;
            }

            // Update result
            reached = Math.Abs(a * Math.Sin(theta * n / 2.0) / Math.Sin(theta / 2.0) - d) <= tolerance;
            effort = Math.Abs((n - 1) * theta) + links[0].Angle;
            distance = (target.Point - new Vector(n * a, 0)).Length();




            target.Result = new Result(reachable, reached, iterations, effort, distance, netEffort);
        }

        public override Chain Copy()
        {
            var clone = new ARCChain(0);
            clone.links = links.Select(i => new Link(i.Length, i.Angle)).ToArray();
            clone.origins = origins.Select(i => new Vector(i.X, i.Y)).ToArray();
            return clone;
        }
    }
}
