﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace IK
{
    class CCDChain : Chain
    {
        public CCDChain(int count)
            : base(count)
        { }


        public override void Move(Target target, int max_iterations, double tolerance)
        {
            bool reachable = false;
            bool reached = false;
            int iterations = 0;
            double effort = 0;
            double distance = 0;
            double netEffort = 0;


            Vector end, last_end;


            reachable = (links.Sum(f => f.Length)) + tolerance >= Length();


            end = Update();


            // if close enough then return
            reached = (target.Point - end).Length() <= tolerance;
            if (!reached) {
                while (true) {

                    // Iterate over the joints
                    iterations++;
                    last_end = end;
                    for (int i = links.Length - 1; i >= 0; i--) {
                        Vector v1 = end - origins[i];
                        Vector v2 = target.Point - origins[i];
                        double rotation = Vector.AngleBetweenWithDirection(v1, v2);

                        links[i].Angle += rotation;
                        effort += Math.Abs(rotation);
                        end = Vector.RotateAbout(end, origins[i], rotation);
                    }
                    distance += (last_end - end).Length();

                    // if close enough then return
                    reached = (target.Point - end).Length() <= tolerance;
                    if (reached)
                        break;

                    // if tried enough then return
                    if (iterations == max_iterations)
                        break;

                    end = Update();
                }
            }

            netEffort = links.Sum(f => Math.Abs(f.Angle));

            target.Result = new Result(reachable, reached, iterations, effort, distance, netEffort);
        }

        public override Chain Copy()
        {
            var clone = new CCDChain(0);
            clone.links = links.Select(i => new Link(i.Length, i.Angle)).ToArray();
            clone.origins = origins.Select(i => new Vector(i.X, i.Y)).ToArray();
            return clone;
        }
    }
}
