﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace IK
{
    abstract class Chain
    {
        protected Link[] links;
        protected Vector[] origins;


        public Chain(int count)
        {
            links = new Link[count];
            origins = new Vector[count];
            for (int i = 0; i < count; i++) {
                links[i] = new Link(1.0 / (double)count, 0);
                origins[i] = new Vector(0, 0);
            }
        }


        public Link[] Links
        {
            get { return links; }
        }


        /// <summary>
        /// Resets the angles of all the links.
        /// </summary>
        public void Reset()
        {
            foreach (var link in links) {
                link.Angle = 0;
            }
        }

        // Computes the total length of all the links.
        public double Length()
        {
            return links.Sum(f => f.Length);
        }

        /// <summary>
        /// Updates the chain and returns the location of the end-effector.
        /// </summary>
        /// <returns></returns>
        public Vector Update()
        {
            double datum = 0;                   // default dataum angle (0 = 3 o'clock)
            Vector origin = new Vector(0, 0);   // default base origin (0,0)

            for (int i = 0; i < links.Length; i++) {
                origins[i] = origin;
                datum += links[i].Angle;
                origin += Vector.FromPolar(links[i].Length, datum);
            }

            return origin;
        }


        public abstract void Move(Target target, int max_iterations, double tolerance);

        public abstract Chain Copy();
    }
}
