﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.Diagnostics;
using System.Threading;

namespace IK
{
    public partial class ImageViewer : Form
    {
        public ImageViewer()
        {
            InitializeComponent();
        }


        /// <summary>
        /// Show the image in a dialog form.
        /// </summary>
        public static void ShowImageAsDialog(Image image)
        {
            using (ImageViewer dialog = new ImageViewer()) {
                dialog.imageBox1.Image = image;
                dialog.ClientSize = image.Size;
                dialog.ShowDialog();
            }
        }


        /// <summary>
        /// Show the image in a form, on a seperate thread.
        /// </summary>
        /// <param name="image"></param>
        public static void ShowImageAsForm(Image image)
        {
            Thread thread = new Thread(delegate()
            {
                ShowImageAsDialog(image);
            });
            thread.IsBackground = true; // will not stop this application from terminating
            thread.SetApartmentState(ApartmentState.STA); // required in a console application
            thread.Start();
        }


        private void saveImageAsStripMenuItem_Click(object sender, EventArgs e)
        {
            if (imageBox1.Image != null) {
                using (SaveFileDialog dialog = new SaveFileDialog()) {
                    dialog.AddExtension = true;
                    dialog.Filter = "Bitmap (*.bmp)|*.bmp|Portable Network Graphics (*.png)|*.png|Jpg (*.jpg)|*.jpg";
                    if (dialog.ShowDialog() == DialogResult.OK) {
                        ImageFormat format;
                        switch (dialog.FilterIndex) {
                            case 1: format = ImageFormat.Bmp; break;
                            case 2: format = ImageFormat.Png; break;
                            case 3: format = ImageFormat.Jpeg; break;
                            default: throw new ApplicationException();
                        }
                        imageBox1.Image.Save(dialog.FileName, format);
                    }
                }
            }
        }
    }
}
