using System.Runtime.InteropServices;

namespace System
{
    /// <summary>
    /// Performance Timer.
    /// </summary>
    public class PerformanceTimer
    {
        [DllImport("Kernel32.dll")]
        private static extern bool QueryPerformanceCounter(out long lpPerformanceCount);

        [DllImport("Kernel32.dll")]
        private static extern bool QueryPerformanceFrequency(out long lpFrequency);

        private bool hasTimer;
        private long startTime;
        private long stopTime;
        private long freq;

        /// <summary>
        /// Has a Hardware Performance Timer.
        /// </summary>
        public bool HasTimer
        {
            get { return hasTimer; }
        }

        /// <summary>
        /// Frequency of CPU.
        /// </summary>
        public long Frequency
        {
            get { return freq; }
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        public PerformanceTimer()
        {
            startTime = 0;
            stopTime = 0;
            freq = 0;
            hasTimer = QueryPerformanceFrequency(out freq);
        }

        /// <summary>
        /// Start the timer.
        /// </summary>
        public void Start()
        {
            if (hasTimer) {
                QueryPerformanceCounter(out startTime);
            }
        }

        /// <summary>
        /// Stop the timer.
        /// </summary>
        public void Stop()
        {
            if (hasTimer) {
                QueryPerformanceCounter(out stopTime);
            }
        }

        /// <summary>
        /// Duration of the timer (in seconds).
        /// </summary>
        public double Duration
        {
            get
            {
                if (hasTimer) {
                    return (double)(stopTime - startTime) / (double)freq;
                } else {
                    return 0;
                }
            }
        }
    }
}