﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace IK
{
    class Workspace
    {
        private int size;
        private Target[][] targets;


        public Workspace(int size)
        {
            this.size = size;
        }


        public int Size
        {
            get { return size; }
            set { size = value; }
        }


        public Target getTarget(int x, int y)
        {
            return targets[y][x];
        }

        private void setTarget(int x, int y, Target target)
        {
            targets[y][x] = target;
        }


        public void Cover(Chain chain, int max_iterations, double tolerance)
        {
            //Cover(chain, max_iterations, tolerance, 1);
            Cover(chain, max_iterations, tolerance, Environment.ProcessorCount);
        }

        public void Cover(Chain chain, int max_iterations, double tolerance, int num_workers)
        {

            // Create the targets
            targets = new Target[size][];
            for (int i = 0; i < size; i++)
                targets[i] = new Target[size];


            // Compute the plot parameters
            double length = chain.Length();
            double radius = (size - 1) / 2.0;
            double scale = length / radius;


            // Generate a list of workers (one for each processor)
            var workers = new Chain[num_workers];
            for (int i = 0; i < workers.Length; i++)
                workers[i] = chain.Copy();


            // Generate a list of jobs
            var jobs = new int[size];
            for (int i = 0; i < jobs.Length; i++)
                jobs[i] = i;


            //for (int y = 0; y < size; y++) {
            Parallel.RunThread(delegate(Chain c, int y)
            {
                for (int x = 0; x < size; x++) {

                    // Generate the target
                    Target target = new Target((x - radius) * scale, (y - radius) * scale);
                    setTarget(x, y, target);

                    // Reset the chain
                    c.Reset();

                    // Solve the chain
                    //result.Reachable = target.Length() <= length + tolerance;
                    c.Move(targets[y][x], max_iterations, tolerance);
                }
            }, workers, jobs);


            //// Run in parallel
            //Parallel.RunThread(delegate(Link[] links, int y)
            //{
            //    for (int x = 0; x < size; x++) {

            //        // Generate the target
            //        Vector target = new Vector(x - radius, y - radius) * scale;

            //        // Reset the chain
            //        Link.ResetAngles(links);

            //        // Solve the chain
            //        Result result = solver.Solve(links, target, max_iterations, tolerance);
            //        result.Reachable = target.Length() <= length + tolerance;

            //        // Store the result in the buffer
            //        buffer[y][x] = result;

            //    }
            //}, workers, jobs);

            //return buffer;
        }
    }
}
